import { Observable } from 'rxjs';
import { Conference, PermissionManager } from '../Sippo';
import { Conferences as Rooms } from '../wac-proxy/stacks/Janus/Conferences';
import { ConferenceLogService } from '../wac-proxy/wac-stack/conference-log/ConferenceLogService';
import { ConferenceLog } from './ConferenceLog';
/**
 * This class provides access to every conference related feature
 * This class must never be directly instantiated, instances of this class
 * can be obtained using Session#getConferenceManager.
 */
export declare class ConferenceManager {
    private conferenceService;
    private conferenceLogService;
    private permissionManager;
    private ownId;
    incomingConference$: Observable<Conference>;
    conferences$: Observable<readonly Conference[]>;
    private conferenceLog?;
    private defaultMediaConstraints;
    private localCreatedConference;
    private subscription;
    /** @ignore */
    constructor(conferenceService: Rooms, conferenceLogService: ConferenceLogService, permissionManager: PermissionManager, ownId: string);
    destroy(): void;
    /**
     * Returns current media constraints used by default when creating conferences
     */
    getDefaultMediaConstraints(): MediaStreamConstraints;
    /**
     * Sets the constraints that will be used as initial value for media requests when
     * initializing conferences. For more information about valid values see
     * {@link https://developer.mozilla.org/en-US/docs/Web/API/MediaStreamConstraints}
     * @example <caption>Enabling noise reduction techniques</caption>
     * conferenceManager.setDefaultMediaConstraints({
     *   audio: {
     *     autoGainControl: true,
     *     echoCancellation: true,
     *     noiseSuppression: true,
     *   },
     *   video: true,
     * });
     * @param {MediaStreamConstraints} defaultMediaConstraints
     */
    setDefaultMediaConstraints(defaultMediaConstraints: MediaStreamConstraints): void;
    /**
     * Creates a new conference room
     * @param options
     * @param options.recording boolean indicating whether the conference is going to be recorded or not. This applies
     * only if the user has {@link PermissionManager.canSetRecordOption} permission
     */
    createConference(options?: {
        recording?: boolean;
    }): Promise<Conference>;
    /**
     * Returns a ConferenceLog reference
     */
    getConferenceLog(): ConferenceLog;
    /**
     * Asks the server for the current invitations available to the user
     */
    getCurrentInvitations(): Promise<Conference[]>;
    /**
     * Get a conference for a meeting
     *
     * @param meetingId
     * @param password
     * @returns fulfilled when the response is received
     * @throws {AccessBeforeTimeError} thrown when trying to access the conference before scheduled time
     * @throws {WrongPasswordError} thrown when the conference is protected by password and the password is not recived or is not valid.
     */
    getMeetingConference(meetingId: string, password?: string): Promise<Conference>;
    /**
     * Gets a quick conference
     *
     * @param quickConferenceId
     * @returns fulfilled when the response is received
     *
     */
    getQuickConference(quickConferenceId: string): Promise<Conference>;
    private getConferenceForRoom;
    private getConferenceForIncomingRoom;
    private getActions;
}
