import {EventEmitter2} from 'eventemitter2';
import * as Immutable from 'immutable';

import {ChatManager} from '../chat/ChatManager';
import {ConferenceManager} from '../conferences/ConferenceManager';
import {ContactRepository} from '../contacts/ContactRepository';
import {DataPipe} from '../datapipe/DataPipe';
import {FileManager} from '../file-sharing/FileManager';
import {GroupRepository} from '../groups/GroupRepository';
import {MeetingManager} from '../meetings/MeetingManager';
import {PermissionManager} from '../permissions/PermissionManager';
import {Presence} from '../users/Presence';
import {UserRepository} from '../users/UserRepository';
import {VoiceMailManager} from '../voice-mail/VoiceMailManager';
import {Whiteboard} from '../whiteboard/Whiteboard';

import {SessionConfig} from './SessionConfig';
import {SessionStatus} from './SessionStatus';
import {User} from './User';

/**
 * Session objects manage communications for a given user identity. They are
 * obtained by calling the createSession method of the Sippo object.
 */
export class Session extends EventEmitter2 {
	static withOAuth2(issuer: string, token: string, config: SessionConfig): Session;
	static withoutToken(issuer: string, config: SessionConfig): Session;
	static withSessionToken(token: string, config: SessionConfig): Session;

	/**
	 * Activates the communications session with a gateway server.
	 */
	connect(): Promise<Session>;

	/**
	 * Creates a new data pipe instance for data exchange with the specified
	 * recipient.
	 */
	createDataPipe(participants: string[], label?: string): DataPipe;

	/**
	 * Creates a new instance of a Selfie.
	 */
	createSelfie(to: string): Promise<any>;

	/***
	 * Creates a new instance of a Whiteboard.
	 */
	createWhiteboard(participants: string[]): Promise<Whiteboard>;

	/**
	 * Destroy every existent token for the deviceId of this session.
	 */
	destroyPushToken(): any;

	/**
	 * Ends a connected session.
	 */
	disconnect(): Promise<void>;

	/**
	 * Get user's alias.
	 */
	getAlias(): string;

	/**
	 * Retrieves an array with the underlying capabilities.
	 */
	getCapabilities(): Immutable.Set<string>;

	/**
	 * Retrieves the chat manager.
	 */
	getChatManager(): Promise<ChatManager>;

	/**
	 * Retrieves a conference manager instance.
	 */
	getConferenceManager(): ConferenceManager;

	getContactRepository(): ContactRepository;

	/**
	 * Retrieves the file manager.
	 */
	getFileManager(): Promise<FileManager>;

	/**
	 * Returns the username in the gateway of the user owning this session.
	 */
	getGatewayUsername(): string;

	getGroupRepository(): GroupRepository;

	/**
	 * Retrieves a Logger instance to log events in the WAC.
	 * @param name The name of the logger
	 * @return A logger instance
	 */
	getLogger(name: string): any;

	getMeetingManager(): MeetingManager;

	/**
	 * Retrieves the permission Manager.
	 */
	getPermissionManager(): PermissionManager;

	/**
	 * Retrieves the voicemail manager
	 */
	getVoiceMailManager(): VoiceMailManager;

	/**
	 * Get session token.
	 */
	getSessionToken(): string;

	/**
	 * Retrieves the current status of this session.
	 */
	getStatus(): SessionStatus;

	/**
	 * Retrieve the User.
	 */
	getUser(): User;

	getPresence(): Presence;

	getUserRepository(): UserRepository;

	/**
	 * Returns the username of the user owning this session.
	 */
	getUsername(): string;

	/**
	 * Retrieves and array with version information.
	 */
	getVersion(): object[];

	/**
	 * Notifies a push token to the WAC.
	 */
	notifyPushToken(tokenType: string, token: string): any;

	onConnectionLost(): void;

	onSessionReconnected(): void;

	onSessionReconnecting(): void;

	onTransportClose(): void;

	onUnknownClose(): void;

	/**
	 * Resume current session, restoring WAC communication.
	 */
	resume(): any;

	/**
	 * @param {String} key Key to modified
	 * @param {String} value New value
	 * @return {Session}
	 */
	setSetting(key: string, value: string): Session;

	/**
	 * Retrieves a promise which is resolved with the settings array.
	 */
	settings(): Promise<any>;

	/**
	 * Suspend current session: WAC will not send or receive any message.
	 */
	suspend(): Promise<any>;
}
